@@ -14,6 +14,8 @@
 package rules
 
 import (
+	"context"
+	"fmt"
 	"testing"
 	"time"
 
@@ -21,11 +23,13 @@ import (
 	"github.com/prometheus/prometheus/pkg/labels"
 	"github.com/prometheus/prometheus/pkg/timestamp"
 	"github.com/prometheus/prometheus/promql"
+	"github.com/prometheus/prometheus/promql/parser"
+	"github.com/prometheus/prometheus/util/teststorage"
 	"github.com/prometheus/prometheus/util/testutil"
 )
 
 func TestAlertingRuleHTMLSnippet(t *testing.T) {
-	expr, err := promql.ParseExpr(`foo{html="<b>BOLD<b>"}`)
+	expr, err := parser.ParseExpr(`foo{html="<b>BOLD<b>"}`)
 	testutil.Ok(t, err)
 	rule := NewAlertingRule("testrule", expr, 0, labels.FromStrings("html", "<b>BOLD</b>"), labels.FromStrings("html", "<b>BOLD</b>"), nil, false, nil)
 
@@ -41,6 +45,46 @@ annotations:
 	testutil.Assert(t, want == got, "incorrect HTML snippet; want:\n\n|%v|\n\ngot:\n\n|%v|", want, got)
 }
 
+func TestAlertingRuleState(t *testing.T) {
+	tests := []struct {
+		name   string
+		active map[uint64]*Alert
+		want   AlertState
+	}{
+		{
+			name: "MaxStateFiring",
+			active: map[uint64]*Alert{
+				0: {State: StatePending},
+				1: {State: StateFiring},
+			},
+			want: StateFiring,
+		},
+		{
+			name: "MaxStatePending",
+			active: map[uint64]*Alert{
+				0: {State: StateInactive},
+				1: {State: StatePending},
+			},
+			want: StatePending,
+		},
+		{
+			name: "MaxStateInactive",
+			active: map[uint64]*Alert{
+				0: {State: StateInactive},
+				1: {State: StateInactive},
+			},
+			want: StateInactive,
+		},
+	}
+
+	for i, test := range tests {
+		rule := NewAlertingRule(test.name, nil, 0, nil, nil, nil, true, nil)
+		rule.active = test.active
+		got := rule.State()
+		testutil.Assert(t, test.want == got, "test case %d unexpected AlertState, want:%d got:%d", i, test.want, got)
+	}
+}
+
 func TestAlertingRuleLabelsUpdate(t *testing.T) {
 	suite, err := promql.NewTest(t, `
 		load 1m
@@ -51,7 +95,7 @@ func TestAlertingRuleLabelsUpdate(t *testing.T) {
 
 	testutil.Ok(t, suite.Run())
 
-	expr, err := promql.ParseExpr(`http_requests < 100`)
+	expr, err := parser.ParseExpr(`http_requests < 100`)
 	testutil.Ok(t, err)
 
 	rule := NewAlertingRule(
@@ -135,7 +179,7 @@ func TestAlertingRuleLabelsUpdate(t *testing.T) {
 				filteredRes = append(filteredRes, smpl)
 			} else {
 				// If not 'ALERTS', it has to be 'ALERTS_FOR_STATE'.
-				testutil.Equals(t, smplName, "ALERTS_FOR_STATE")
+				testutil.Equals(t, "ALERTS_FOR_STATE", smplName)
 			}
 		}
 
@@ -153,7 +197,7 @@ func TestAlertingRuleExternalLabelsInTemplate(t *testing.T) {
 
 	testutil.Ok(t, suite.Run())
 
-	expr, err := promql.ParseExpr(`http_requests < 100`)
+	expr, err := parser.ParseExpr(`http_requests < 100`)
 	testutil.Ok(t, err)
 
 	ruleWithoutExternalLabels := NewAlertingRule(
@@ -214,7 +258,7 @@ func TestAlertingRuleExternalLabelsInTemplate(t *testing.T) {
 			filteredRes = append(filteredRes, smpl)
 		} else {
 			// If not 'ALERTS', it has to be 'ALERTS_FOR_STATE'.
-			testutil.Equals(t, smplName, "ALERTS_FOR_STATE")
+			testutil.Equals(t, "ALERTS_FOR_STATE", smplName)
 		}
 	}
 
@@ -228,7 +272,7 @@ func TestAlertingRuleExternalLabelsInTemplate(t *testing.T) {
 			filteredRes = append(filteredRes, smpl)
 		} else {
 			// If not 'ALERTS', it has to be 'ALERTS_FOR_STATE'.
-			testutil.Equals(t, smplName, "ALERTS_FOR_STATE")
+			testutil.Equals(t, "ALERTS_FOR_STATE", smplName)
 		}
 	}
 
@@ -245,7 +289,7 @@ func TestAlertingRuleEmptyLabelFromTemplate(t *testing.T) {
 
 	testutil.Ok(t, suite.Run())
 
-	expr, err := promql.ParseExpr(`http_requests < 100`)
+	expr, err := parser.ParseExpr(`http_requests < 100`)
 	testutil.Ok(t, err)
 
 	rule := NewAlertingRule(
@@ -284,8 +328,41 @@ func TestAlertingRuleEmptyLabelFromTemplate(t *testing.T) {
 			filteredRes = append(filteredRes, smpl)
 		} else {
 			// If not 'ALERTS', it has to be 'ALERTS_FOR_STATE'.
-			testutil.Equals(t, smplName, "ALERTS_FOR_STATE")
+			testutil.Equals(t, "ALERTS_FOR_STATE", smplName)
 		}
 	}
 	testutil.Equals(t, result, filteredRes)
 }
+
+func TestAlertingRuleDuplicate(t *testing.T) {
+	storage := teststorage.New(t)
+	defer storage.Close()
+
+	opts := promql.EngineOpts{
+		Logger:     nil,
+		Reg:        nil,
+		MaxSamples: 10,
+		Timeout:    10 * time.Second,
+	}
+
+	engine := promql.NewEngine(opts)
+	ctx, cancelCtx := context.WithCancel(context.Background())
+	defer cancelCtx()
+
+	now := time.Now()
+
+	expr, _ := parser.ParseExpr(`vector(0) or label_replace(vector(0),"test","x","","")`)
+	rule := NewAlertingRule(
+		"foo",
+		expr,
+		time.Minute,
+		labels.FromStrings("test", "test"),
+		nil,
+		nil,
+		true, log.NewNopLogger(),
+	)
+	_, err := rule.Eval(ctx, now, EngineQueryFunc(engine, storage), nil)
+	testutil.NotOk(t, err)
+	e := fmt.Errorf("vector contains metrics with the same labelset after applying alert labels")
+	testutil.ErrorEqual(t, e, err)
+}
